#!/usr/bin/env python

import rospy
import actionlib
import adc_msgs.msg
from std_msgs.msg import String
from random import randrange

class TakeoffAction(object):
    _feedback = adc_msgs.msg.ADCTakeoffFeedback()
    _result = adc_msgs.msg.ADCTakeoffResult()

    def __init__(self, name):
        self._action_name = name
        self._as = actionlib.SimpleActionServer(self._action_name, adc_msgs.msg.ADCTakeoffAction, execute_cb=self.execute_cb, auto_start = False)
        self._as.start()
      
    def execute_cb(self, goal):
        # helper variables
        r = rospy.Rate(1)
        success = True
      
        # start executing the action
        limit = randrange(200000)
        print(limit)
        for i in range(1, 5):
            #simulate some random failures
            if i > limit:
                self._as.set_aborted()
                success = False
                break

            # check if client canceled the action    
            if self._as.is_preempt_requested():
                rospy.loginfo('%s: Preempted' % self._action_name)
                self._as.set_preempted()
                success = False
                break

            # publish the feedback, if successful
            self._feedback.status_code = String("takeoff still going:" + str(i))
            self._as.publish_feedback(self._feedback)
            rospy.loginfo(str(i))
            r.sleep()
          
        if success:
            rospy.loginfo('%s: Succeeded' % self._action_name)
            self._result.return_code = String("done")
            self._as.set_succeeded(self._result)

class LandAction(object):
    _feedback = adc_msgs.msg.ADCLandFeedback()
    _result = adc_msgs.msg.ADCLandResult()

    def __init__(self, name):
        self._action_name = name
        self._as = actionlib.SimpleActionServer(self._action_name, adc_msgs.msg.ADCLandAction, execute_cb=self.execute_cb, auto_start = False)
        self._as.start()
      
    def execute_cb(self, goal):
        # helper variables
        r = rospy.Rate(1)
        success = True
      
        # start executing the action
        limit = randrange(200000)
        print(limit)
        for i in range(1, 5):
            #simulate some random failures
            if i > limit:
                self._as.set_aborted()
                success = False
                break

            if self._as.is_preempt_requested():
                rospy.loginfo('%s: Preempted' % self._action_name)
                self._as.set_preempted()
                success = False
                break
            # publish the feedback
            self._feedback.status_code = String("land still going:" + str(i))
            self._as.publish_feedback(self._feedback)
            rospy.loginfo(str(i))
            r.sleep()
          
        if success:
            rospy.loginfo('%s: Succeeded' % self._action_name)
            self._result.return_code = String("done")
            self._as.set_succeeded(self._result)

class FlyToPOIAction(object):
    _feedback = adc_msgs.msg.ADCFlyToPOIFeedback()
    _result = adc_msgs.msg.ADCFlyToPOIResult()

    def __init__(self, name):
        self._action_name = name
        self._as = actionlib.SimpleActionServer(self._action_name, adc_msgs.msg.ADCFlyToPOIAction, execute_cb=self.execute_cb, auto_start = False)
        self._as.start()
      
    def execute_cb(self, goal):
        rospy.loginfo("going to " + goal.poi.data + " at (" + str(goal.x) + "," + str(goal.y) + "," + str(goal.z) + "," + str(goal.w) + ")")
        # helper variables
        r = rospy.Rate(1)
        success = True
      
        # start executing the action
        limit = randrange(200000)
        print(limit)
        for i in range(1, 5):
            #simulate some random failures
            if i > limit:
                self._as.set_aborted()
                success = False
                break

            if self._as.is_preempt_requested():
                rospy.loginfo('%s: Preempted' % self._action_name)
                self._as.set_preempted()
                success = False
                break
            # publish the feedback
            self._feedback.status_code = String("FlyToPOI still going:" + str(i))
            self._as.publish_feedback(self._feedback)
            rospy.loginfo(str(i))
            r.sleep()
          
        if success:
            rospy.loginfo('%s: Succeeded' % self._action_name)
            self._result.return_code = String("done")
            self._as.set_succeeded(self._result)


class YawAction(object):
    _feedback = adc_msgs.msg.ADCYawFeedback()
    _result = adc_msgs.msg.ADCYawResult()

    def __init__(self, name):
        self._action_name = name
        self._as = actionlib.SimpleActionServer(self._action_name, adc_msgs.msg.ADCYawAction, execute_cb=self.execute_cb, auto_start = False)
        self._as.start()
      
    def execute_cb(self, goal):
        rospy.loginfo("yawing to " + str(goal.angle))
        # helper variables
        r = rospy.Rate(1)
        success = True
      
        # start executing the action
        limit = randrange(200000)
        print(limit)
        for i in range(1, 5):
            #simulate some random failures
            if i > limit:
                self._as.set_aborted()
                success = False
                break

            if self._as.is_preempt_requested():
                rospy.loginfo('%s: Preempted' % self._action_name)
                self._as.set_preempted()
                success = False
                break
            # publish the feedback
            self._feedback.status_code = String("Yaw still going:" + str(i))
            self._as.publish_feedback(self._feedback)
            rospy.loginfo(str(i))
            r.sleep()
          
        if success:
            rospy.loginfo('%s: Succeeded' % self._action_name)
            self._result.return_code = String("done")
            self._as.set_succeeded(self._result)

        
if __name__ == '__main__':
    rospy.init_node('autonomy')
    takeoff_server = TakeoffAction('takeoff')
    land_server = LandAction('land')
    flytopoi_server = FlyToPOIAction('fly_to_poi')
    yaw_server = YawAction('yaw')
    rospy.spin()