// https://community.plotly.com/t/obtaining-the-data-coordinates-of-the-mouse/11761/3
// refPlotly.syncWindowResize

// modebar buttons edit
// https://plotly.com/javascript/configuration-options/

// docs
// https://plotly.com/javascript/reference/layout/images/

class ComponentMap extends React.Component {
    constructor(props) {
        super(props);

        this.state = {
            width: 0,
            height: 0,
        }

        this.point_hover = {x: 0, y: 0};
        this.active_poi = false;
        this.active_poi_uuid = "";

        if(props.project) {
            let poi_active = props.project.map_layers.layer_pois.pois.find(poi => poi.is_selected);
            if(poi_active) {
                this.active_poi = poi_active;
                this.active_poi_uuid = poi_active.uuid;
            }
        }

        this.refPlotly = React.createRef();
        this.timeLastClick = (new Date()).getTime();

        this.gd = false;

        this.drone_position = {
            x: 0,
            y: 0,
            dir_x: 0,
            dir_y: 0,
            angle: 0,
            is_set: false
        };

        document.addEventListener(UPDATE_DRONE_POSITION, this.onSetDronePosition);

        document.addEventListener(POI_CLEAR_SELECTION, this.clearSelection);
        document.addEventListener(POI_SAVE_ACTIVE, this.onSaveActive);
        document.addEventListener(POI_DELETE_ACTIVE, this.onDeleteActive);
        document.addEventListener(POI_DELETE_ALL, this.onDeleteAll);
        document.addEventListener(POI_FLY_TO_ACTIVE, this.onFlyToActive);
        document.addEventListener(POI_SET_SELECTION, this.onPOISetSelection);
    }

    onPOISetSelection = (event) => {
        this.active_poi_uuid = event.detail;
        this.active_poi = this.getSelectedPOI();
        this.updateAnnotations();
    }

    getSelectedPOI = () => {
        let poi = false;
        if (this.props.project && this.active_poi_uuid) {
            poi = this.props.project.map_layers.layer_pois.pois.find(poi => poi.uuid === this.active_poi_uuid);
        }
        return poi;
    }

    onFlyToActive = (event) => {
        if (this.active_poi) {
            let poi = this.getSelectedPOI();
            let active_poi_name = "selected POI";
            if(poi) {
                active_poi_name = poi.name;
            }
            eel.adc_fly_to_poi(
                active_poi_name,
                this.active_poi.x,
                this.active_poi.y,
            )
        }
    }

    onSaveActive = (event) => {
        try {
            if (this.active_poi) {
                let poi_uuid = "";
                let poi = this.getSelectedPOI();
                if(poi) {
                    poi_uuid = poi.uuid;
                }
                eel.save_poi(event.detail, this.active_poi.x, this.active_poi.y, poi_uuid);
                this.clearSelection();
            }
        }
        catch (e) {
            alert(e);
        }
    }

    onDeleteActive = (event) => {
        try {
            if(confirm("Are you sure you want to delete the selected POI?")) {
                if (this.props.project) {
                    let poi = this.getSelectedPOI();
                    if(poi) {
                        eel.delete_poi(poi.uuid);
                        this.clearSelection();
                    }
                }
            }

        }
        catch (e) {
            alert(e);
        }
    }

    onDeleteAll = (event) => {
        try {
            if(confirm("Are you sure you want to delete all POIs?")) {
                this.clearSelection();
                eel.clear_all_poi();
            }
        }
        catch (e) {
            alert(e);
        }
    }

    onSetDronePosition = (event) => {
        this.drone_position.is_set = true;
        this.drone_position.x = event.detail.x;
        this.drone_position.y = event.detail.y;
        this.drone_position.dir_x = event.detail.dir_x;
        this.drone_position.dir_y = event.detail.dir_y;
        this.drone_position.angle = event.detail.angle;
        this.updateAnnotations();
    }

    onResize = () => {
        let jPlot = $('.plotly .svg-container');
        jPlot.hide();
        let jPanelMain = $('#panel-main-plot');
        let width = parseInt(jPanelMain.width(), 10);
        let height = parseInt(jPanelMain.height(), 10);
        jPlot.show();
        if (this.state.width !== width || this.state.height !== height) {
            this.setState({
                width: width,
                height: height
            });
        }
    };

    clearSelection = () => {
        this.active_poi = false;
        this.updateAnnotations();
    }

    componentDidMount = () => {
        this.onResize();

        let gd = this.refPlotly.current.el;
        let jgd = $(gd);
        this.jgd = jgd;
        this.gd = gd;

        jgd.on('click', this.onClick)
        jgd.on('mousemove', this.onMouseMove)
    }

    onInitialized = (event) => {
        console.log('onInitialized', event, this.refPlotly.current);
    }

    onMouseMove = (event) => {
        let relX = event.pageX - this.jgd.offset().left;
        let relY = event.pageY - this.jgd.offset().top;
        let gd = this.gd;
        let xaxis = gd._fullLayout.xaxis;
        let yaxis = gd._fullLayout.yaxis;
        let l = gd._fullLayout.margin.l;
        let t = gd._fullLayout.margin.t;
        this.point_hover.x = xaxis.p2c(relX - l);
        this.point_hover.y = yaxis.p2c(relY - t);
    }

    // Adding POIs
    onClick = (event) => {
        if (this.props.project.view_settings.is_layer_poi &&
            this.props.project.map_layers.layer_pointcloud_raw.x.length > 0
        ) {
            let timeNow = (new Date()).getTime();
            if (timeNow - this.timeLastClick < 1000) {

                if (this.props.project.view_settings.is_layer_flight_path) {
                    if (this.props.project.view_settings.is_snap_to_flight_path_poi) {
                        let minDist = 1e9;
                        let minIdx = -1;
                        for (let idx = 0; idx < this.props.project.map_layers.layer_drone_path.x.length; idx++) {
                            let x = this.props.project.map_layers.layer_drone_path.x[idx];
                            let y = this.props.project.map_layers.layer_drone_path.y[idx];
                            let dist = Math.pow(this.point_hover.x - x, 2) + Math.pow(this.point_hover.y - y, 2);
                            if (minDist > dist) {
                                minDist = dist;
                                minIdx = idx;
                            }
                        }
                        if (minIdx > 0) {
                            this.point_hover.x = this.props.project.map_layers.layer_drone_path.x[minIdx];
                            this.point_hover.y = this.props.project.map_layers.layer_drone_path.y[minIdx];
                        }
                    }
                }


                this.active_poi = {
                    x: this.point_hover.x,
                    y: this.point_hover.y,
                };

                this.updateAnnotations();

                eel.change_poi("", this.active_poi.x, this.active_poi.y)
            }
            this.timeLastClick = timeNow
        }
    }

    updateAnnotations = () => {
        Plotly.update(this.gd, {}, {
            images: this.getLayoutImages(),
            annotations: this.getLayoutAnnotations()
        })
    }

    getLayoutImages = () => {
        let images = [];
        if (this.props.project.view_settings.is_layer_poi) {

            let activePOI = false;
            let pois = [...this.props.project.map_layers.layer_pois.pois];

            let xaxis = null;
            let yaxis = null;
            let l = 0;
            let t = 0;
            if(this.gd && this.gd._fullLayout) {
                let gd = this.gd;
                xaxis = gd._fullLayout.xaxis;
                yaxis = gd._fullLayout.yaxis;
                l = gd._fullLayout.margin.l;
                t = gd._fullLayout.margin.t;
            }

            for (let poi of pois) {

                if(xaxis) {
                    let dist = Math.hypot(
                        xaxis.c2p(this.active_poi.x + l) - xaxis.c2p(poi.x + l) ,
                        yaxis.c2p(this.active_poi.y + t) - yaxis.c2p(poi.y + t));

                    if(dist < 26) {
                        activePOI = poi;
                        if(poi.uuid !== this.active_poi_uuid) {
                            this.active_poi_uuid = poi.uuid;
                            this.active_poi.x = poi.x;
                            this.active_poi.y = poi.y;
                            eel.change_poi(this.active_poi_uuid, this.active_poi.x, this.active_poi.y);
                        }
                    }
                }

                images.push({
                    x: poi.x,
                    y: poi.y,
                    sizex: 0.05,
                    sizey: 0.05,
                    sizing: "contain",
                    source: (poi === activePOI ? "images/poi_active.png":  "images/poi_inactive.png"),
                    xanchor: "center",
                    yanchor: "middle",
                    xref: "x domain",
                    yref: "y domain"
                });
            }
            if(!activePOI) {
                if (this.active_poi) {
                    this.active_poi_uuid = "";
                    images.push({
                        x: this.active_poi.x,
                        y: this.active_poi.y,
                        sizex: 0.05,
                        sizey: 0.05,
                        sizing: "contain",
                        source: "images/poi_active.png",
                        xanchor: "center",
                        yanchor: "middle",
                        xref: "x domain",
                        yref: "y domain"
                    });
                }
            }
        }

        if(this.drone_position.is_set) {
            images.push({
                x: this.drone_position.x,
                y: this.drone_position.y,
                sizex: 0.1,
                sizey: 0.1,
                sizing: "contain",
                source: "images/drone.png",
                xanchor: "center",
                yanchor: "middle",
                xref: "x domain",
                yref: "y domain",
                angle: this.drone_position.angle
            });
        }
        return images;
    }

    getLayoutAnnotations = () => {
        let annotations = [];
        if (this.props.project.view_settings.is_layer_poi) {

            let pois = [...this.props.project.map_layers.layer_pois.pois];

            for (let poi of pois) {
                annotations.push({
                    x: poi.x,
                    y: poi.y,
                    text: poi.name,
                    xref: "x",
                    yref: "y",
                    bgcolor: "white",
                    xanchor: "left",
                    yanchor: "top",
                    showarrow: true,
                    arrowcolor: "transparent",
                    ax: 5,
                    ay: -25,
                    axref: "pixel",
                    ayref: "pixel"
                });
            }
        }
        return annotations;
    }

    render = () => {
        let project = this.props.project;
        if (!project) {
            return <div/>;
        }
        let layers = [];

        if (project.view_settings.is_layer_pointcloud_raw) {
            layers.push({
                x: this.props.project.map_layers.layer_pointcloud_raw.x,
                y: this.props.project.map_layers.layer_pointcloud_raw.y,
                type: 'scatter',
                mode: 'markers',
                marker: {
                    color: '#8E8E8E',
                    size: 2
                },
                hoverinfo: "none",
            });
        }

        if (project.view_settings.is_layer_pointcloud_filtered) {
            layers.push(
                {
                    x: this.props.project.map_layers.layer_pointcloud_filtered.x,
                    y: this.props.project.map_layers.layer_pointcloud_filtered.y,
                    type: 'scatter',
                    mode: 'markers',
                    marker: {
                        color: '#FF5656',
                        size: 2
                    },
                    hoverinfo: "none",
                }
            );
        }

        if (project.view_settings.is_layer_walls) {
            let walls = []
            for (let wall_segment of this.props.project.map_layers.layer_walls.walls) {
                walls.push({
                    x: wall_segment.x,
                    y: wall_segment.y,
                    mode: 'lines',
                    line: {
                        color: '#0085FF',
                        width: 4
                    },
                    hoverinfo: "none",
                });
            }
            layers = _.concat(layers, walls);
        }

        if (project.view_settings.is_layer_flight_path) {
            layers.push(
                {
                    x: this.props.project.map_layers.layer_drone_path.x,
                    y: this.props.project.map_layers.layer_drone_path.y,
                    mode: 'lines',
                    line: {
                        color: '#00E224',
                        width: 2,
                        dash: 'dashdot',
                    },
                    hoverinfo: "none",
                }
            );
        }


        return (
            <PlotlyComponent
                data={layers}
                ref={this.refPlotly}
                onInitialized={this.onInitialized}
                layout={{
                    images: this.getLayoutImages(),
                    annotations: this.getLayoutAnnotations(),
                    width: this.state.width,
                    height: this.state.height,
                    paper_bgcolor: '#060D13',
                    plot_bgcolor: '#060D13',
                    showlegend: false,
                    hovermode: "closest",
                    hoverinfo: "none",
                    margin: {
                        l: 0,
                        r: 0,
                        t: 0,
                        b: 0
                    },
                    xaxis: {
                        uirevision: 'x',
                        showgrid: false,
                        zeroline: false,
                        showline: false,
                        visible: false,
                        // range: [
                        //     _.min(this.props.project.map_layers.layer_pointcloud_raw.x),
                        //     _.max(this.props.project.map_layers.layer_pointcloud_raw.x)
                        // ]
                    },
                    yaxis: {
                        uirevision: 'y',
                        showgrid: false,
                        zeroline: false,
                        showline: false,
                        visible: false,
                        scaleanchor: "x",
                        scaleratio: 1,

                        // range: [
                        //     _.min(this.props.project.map_layers.layer_pointcloud_raw.y),
                        //     _.max(this.props.project.map_layers.layer_pointcloud_raw.y)
                        // ]
                    }
                }}
                config={{
                    responsive: true,
                    displayModeBar: true,
                    doubleClick: false
                }}
            />);
    }
}


