class ComponentPanelPOIs extends React.Component {
    constructor(props) {
        super(props);

        let selected_uuid = null;
        if(this.props.project) {
            let selected_pois = this.props.project.map_layers.layer_pois.pois.find(poi => poi.is_selected);
            if(selected_pois) {
                selected_uuid = selected_pois.uuid;
            }
        }

        this.setState({
            selected_uuid: selected_uuid,
        });

        document.addEventListener(OPEN_NEW_FILE, this.openNewFile);
        document.addEventListener(POI_SET_SELECTION, this.poiSetSelection);
    }

    poiSetSelection = (event) => {
        this.setState({
            selected_uuid: event.detail
        });
    }

    openNewFile = () => {
        this.setState({
            poi: 'Select POI'
        });
    }

    onSavePOI = () => {
        let poi_label = prompt('Enter label of POI:').trim();
        if (poi_label.length) {
            document.dispatchEvent(new CustomEvent(POI_SAVE_ACTIVE, {detail: poi_label}));
        }
    }

    onDeletePOI = () => {
        document.dispatchEvent(new CustomEvent(POI_DELETE_ACTIVE, {detail: {}}));
    }

    onClearAllPOIs = () => {
        if (confirm('Are you sure you want to delete all POIs?')) {
            document.dispatchEvent(new CustomEvent(POI_DELETE_ALL, {detail: {}}));
        }
    }

    onAdcFlyToPOI = () => {
        document.dispatchEvent(new CustomEvent(POI_FLY_TO_ACTIVE, {detail: {}}));
    }

    onFlightPlan = () => {
        this.props.onDialogLoadFlightPlan();
    }

    componentDidMount = () => {
    }

    handleChange = (event) => {
        eel.change_poi(event.target.value, 0, 0);
    }

    render = () => {
        let selected_name = "Select POI";
        let selected_uuid = null;
        if (this.state && this.state.selected_uuid) {
            let poi = this.props.project.map_layers.layer_pois.pois.find(poi => poi.uuid === this.state.selected_uuid);
            if (poi) {
                selected_name = poi.name;
                selected_uuid = poi.uuid;
            }
        }
        return (
            <div id="panel-poi-side">
                <FormControlLabel
                    control={<Checkbox
                        checked={this.props.isSnapToFlightPath}
                        onChange={(event) => {
                            this.props.onSnapToFlightPathChange(event);
                        }}
                    />}
                    label="Snap to flight path"
                />
                <div style={{color: "#2E3843", paddingTop: "5px"}}>
                    <FormControl sx={{m: 1, width: 260, mt: 3}}
                                 disabled={this.props.project && this.props.project.map_layers.layer_pois.pois.length > 0 ? undefined : true}>
                        <Select
                            sx={{backgroundColor: "#2E3843", color: "#fff", borderRadius: "10px", "& .MuiSvgIcon-root": {color: "white"}}}
                            displayEmpty
                            value={selected_uuid}
                            onChange={this.handleChange}
                            renderValue={() => {
                                return <>{selected_name}</>
                            }}
                        >
                            {this.props.project.map_layers.layer_pois.pois.map((poi) => (
                                <MenuItem key={poi.uuid} value={poi.uuid}>{poi.name}</MenuItem>
                            ))}
                        </Select>
                    </FormControl>
                </div>
                <Button variant="contained" style={{marginBottom: 10}} onClick={this.onSavePOI}>Save selected
                    POI</Button>
                <Button variant="contained" style={{marginBottom: 10}} onClick={this.onDeletePOI}>Delete selected
                    POI</Button> &nbsp;
                <Button variant="contained" style={{marginBottom: 10}} onClick={this.onClearAllPOIs}>Clear all</Button>
                <Button variant="contained" style={{marginBottom: 10}} onClick={this.onAdcFlyToPOI}>Fly to selected
                    POI</Button> &nbsp;
                <Button variant="contained" style={{marginBottom: 10}} onClick={this.onFlightPlan}>Flight Plan</Button>
                <div className="clear"></div>
                <span className="span-info">* Double click to select or add new POI</span>
            </div>
        );
    }
}


